@@ -6,9 +6,9 @@
  * Written by:
  *	Jan Nijtmans
  *	CMG (Computer Management Group) Arnhem B.V.
- *	email: Jan.Nijtmans@wxs.nl (private)
- *	       Jan.Nijtmans@cmg.nl (work)
- *	url:   http://home.wxs.nl/~nijtmans/
+ *	email: j.nijtmans@chello.nl (private)
+ *	       jan.nijtmans@cmg.nl (work)
+ *	url:   http://purl.oclc.org/net/nijtmans/
  *
  */
 #include "tk.h"
@@ -31,7 +31,7 @@
 #define MAX_WORD_LENGTH 100
 typedef struct ParseInfo {
     MFile handle;
-    char word[MAX_WORD_LENGTH+1];
+    char word[MAX_WORD_LENGTH+2];
 				/* Current word of bitmap data, NULL
 				 * terminated. */
     int wordLength;		/* Number of non-NULL bytes in word. */
@@ -55,12 +55,12 @@ static int	        ObjReadXBM _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *dataObj, Tcl_Obj *format,
               		    Tk_PhotoHandle imageHandle, int destX, int destY,
 		            int width, int height, int srcX, int srcY));
-static int	        StringWriteXBM _ANSI_ARGS_((Tcl_Interp *interp,
-               		    Tcl_DString *dataPtr, Tcl_Obj *format,
-		            Tk_PhotoImageBlock *blockPtr));
 static int              ChnWriteXBM _ANSI_ARGS_((Tcl_Interp *interp,
                             char *fileName, Tcl_Obj *format,
                             Tk_PhotoImageBlock *blockPtr));
+static int	        StringWriteXBM _ANSI_ARGS_((Tcl_Interp *interp,
+               		    Tcl_DString *dataPtr, Tcl_Obj *format,
+		            Tk_PhotoImageBlock *blockPtr));
 
 static int		CommonReadXBM _ANSI_ARGS_((Tcl_Interp *interp,
 			    ParseInfo *parseInfo,
@@ -68,11 +68,11 @@ static int		CommonReadXBM _ANSI_ARGS_((Tcl_Interp *interp,
 			    int destX, int destY, int width, int height,
 			    int srcX, int srcY));
 static int		CommonWriteXBM _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *fileName, Tcl_DString *dataPtr,
+			    CONST char *fileName, Tcl_DString *dataPtr,
 			    Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
 
 Tk_PhotoImageFormat imgFmtXBM = {
-    "XBM",					/* name */
+    "xbm",					/* name */
     ChnMatchXBM,	/* fileMatchProc */
     ObjMatchXBM,	/* stringMatchProc */
     ChnReadXBM,	/* fileReadProc */
@@ -117,6 +117,8 @@ ObjMatchXBM(interp, data, format, widthPtr, heightPtr)
 {
     ParseInfo parseInfo;
 
+    ImgFixObjMatchProc(&interp, &data, &format, &widthPtr, &heightPtr);
+
     parseInfo.handle.data = ImgGetStringFromObj(data, &parseInfo.handle.length);
     parseInfo.handle.state = IMG_STRING;
 
@@ -154,6 +156,8 @@ ChnMatchXBM(interp, chan, fileName, format, widthPtr, heightPtr)
 {
     ParseInfo parseInfo;
 
+    ImgFixChanMatchProc(&interp, &chan, &fileName, &format, &widthPtr, &heightPtr);
+
     parseInfo.handle.data = (char *) chan;
     parseInfo.handle.state = IMG_CHAN;
 
@@ -404,12 +408,15 @@ NextBitmapWord(parseInfoPtr)
 
     for (num=ImgRead(&parseInfoPtr->handle,&buf,1); isspace(UCHAR(buf)) || (buf == ',');
 	    num=ImgRead(&parseInfoPtr->handle,&buf,1)) {
-	if (buf == EOF) {
+	if (num == 0 || buf == EOF) {
 	    return TCL_ERROR;
 	}
     }
     for ( ; !isspace(UCHAR(buf)) && (buf != ',') && (num != 0);
 	    num=ImgRead(&parseInfoPtr->handle,&buf,1)) {
+	if (UCHAR(buf) < ' ' || UCHAR(buf) > '~') {
+	    return TCL_ERROR;
+	}
 	*dst = buf;
 	dst++;
 	parseInfoPtr->wordLength++;
@@ -505,7 +512,6 @@ ReadXBMFileHeader(pi, widthPtr, heightPtr)
 		}
 	    }
 	} else if ((pi->word[0] == '{') && (pi->word[1] == 0)) {
-
 	    return 0;
 	}
     }
@@ -560,14 +566,24 @@ ChnWriteXBM(interp, fileName, format, blockPtr)
  *
  *----------------------------------------------------------------------
  */
-static int	        
-StringWriteXBM(interp, dataPtr, format, blockPtr) 
+static int	
+StringWriteXBM(interp, dataPtr, format, blockPtr)
     Tcl_Interp *interp;
     Tcl_DString *dataPtr;
     Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
-  return CommonWriteXBM(interp, (char *) NULL, dataPtr, format, blockPtr);
+    int result;
+    Tcl_DString data;
+
+    ImgFixStringWriteProc(&data, &interp, &dataPtr, &format, &blockPtr);
+
+    result = CommonWriteXBM(interp, (CONST char *) NULL, dataPtr, format, blockPtr);
+
+    if ((result == TCL_OK) && (dataPtr == &data)) {
+	Tcl_DStringResult(interp, dataPtr);
+    }
+    return result;
 }
 
 
@@ -581,7 +597,7 @@ StringWriteXBM(interp, dataPtr, format, blockPtr)
  *
  * CommonWriteXBM
  *
- *	This procedure writes a XBM image to the file filename 
+ *	This procedure writes a XBM image to the file filename
  *      (if filename != NULL) or to dataPtr.
  *
  * Results:
@@ -595,13 +611,13 @@ StringWriteXBM(interp, dataPtr, format, blockPtr)
 static int
 CommonWriteXBM(interp, fileName, dataPtr, format, blockPtr)
     Tcl_Interp *interp;
-    char *fileName;
+    CONST char *fileName;
     Tcl_DString *dataPtr;
-    Tcl_Obj *format;    
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     Tcl_Channel chan = (Tcl_Channel) NULL;
-    char buffer[256];           
+    char buffer[256];
     unsigned char *pp;
     int x, y, i, value, mask;
     int sep = ' ';
@@ -610,7 +626,7 @@ CommonWriteXBM(interp, fileName, dataPtr, format, blockPtr)
     static CONST char header[] =
 "#define %s_width %d\n\
 #define %s_height %d\n\
-static char %s_bits[] = {\n";  
+static char %s_bits[] = {\n";
 
     alphaOffset = blockPtr->offset[0];
     if (alphaOffset < blockPtr->offset[1]) alphaOffset = blockPtr->offset[1];
@@ -624,7 +640,7 @@ static char %s_bits[] = {\n";
 
     /* open the output file (if needed) */
     if (fileName) {
-      chan = Tcl_OpenFileChannel(interp, fileName, "w", 0644);
+      chan = Tcl_OpenFileChannel(interp, (char *) fileName, "w", 0644);
       if (!chan) {
 	return TCL_ERROR;
       }
@@ -646,16 +662,16 @@ static char %s_bits[] = {\n";
 	}
 	p = strchr(fileName, '.');
 	if (p) {
-	    *p = 0;
+	    *(char *)p = 0;
 	}
     } else {
         fileName = "unknown";
     }
-    
+
     sprintf(buffer, header, fileName, blockPtr->width, fileName,
 	    blockPtr->height, fileName);
     if (p) {
-	*p = '.';   
+	*p = '.';
     }
     WRITE(buffer);
 
@@ -682,7 +698,7 @@ static char %s_bits[] = {\n";
 	      mask = 1;
 	      sep = ',';
              }
-	}          
+	}
 	if (mask != 1) {
 	      sprintf(buffer,"%c 0x%02x",sep, value);
 	      WRITE(buffer);
