@@ -1,20 +1,36 @@
-/* 
+/*
  * tkWinX.c --
  *
- *	This file contains Windows emulation procedures for X routines. 
+ *	This file contains Windows emulation procedures for X routines.
  *
  * Copyright (c) 1995-1996 Sun Microsystems, Inc.
  * Copyright (c) 1994 Software Research Associates, Inc.
+ * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkWinX.c 1.51 97/09/02 13:06:57
+ * RCS: @(#) $Id: tkWinX.c,v 1.4 1998/10/10 00:30:37 rjohnson Exp $
  */
 
 #include "tkInt.h"
 #include "tkWinInt.h"
 
+/*
+ * The zmouse.h file includes the definition for WM_MOUSEWHEEL.
+ */
+
+#ifndef __GNUC__       // not in Minw32 yet
+#ifndef __BORLANDC__
+#include <zmouse.h>
+#endif
+#endif
+#ifndef WM_MOUSEWHEEL
+#define WM_MOUSEWHEEL (WM_MOUSELAST+1)  // message that will be supported
+                                        // by the OS
+#endif
+
+
 /*
  * Definitions of extern variables supplied by this file.
  */
@@ -96,7 +112,7 @@ TkGetServerInfo(interp, tkwin)
  *----------------------------------------------------------------------
  */
 
-static BOOL CALLBACK 
+static BOOL CALLBACK
 FindMyConsole( HWND win, LPARAM arg )
 {
  DWORD mypid = *((DWORD *) arg);
@@ -195,7 +211,7 @@ TkWinXCleanup(hInstance)
     /*
      * Clean up our own class.
      */
-    
+
     if (childClassInitialized) {
         childClassInitialized = 0;
         UnregisterClass(TK_WIN_CHILD_CLASS_NAME, hInstance);
@@ -204,7 +220,7 @@ TkWinXCleanup(hInstance)
     /*
      * And let the window manager clean up its own class(es).
      */
-    
+
     TkWinWmCleanup(hInstance);
 }
 
@@ -289,7 +305,7 @@ TkpOpenDisplay(display_name)
 	    GetDeviceCaps(dc, LOGPIXELSX) * 10);
     screen->mheight = MulDiv(screen->height, 254,
 	    GetDeviceCaps(dc, LOGPIXELSY) * 10);
-    
+
     /*
      * Set up the root window.
      */
@@ -302,10 +318,10 @@ TkpOpenDisplay(display_name)
     twdPtr->window.winPtr = NULL;
     twdPtr->window.handle = NULL;
     screen->root = (Window)twdPtr;
- 
+
     /*
-     * On windows, when creating a color bitmap, need two pieces of 
-     * information: the number of color planes and the number of 
+     * On windows, when creating a color bitmap, need two pieces of
+     * information: the number of color planes and the number of
      * pixels per plane.  Need to remember both quantities so that
      * when constructing an HBITMAP for offscreen rendering, we can
      * specify the correct value for the number of planes.  Otherwise
@@ -403,7 +419,7 @@ TkpCloseDisplay(dispPtr)
     /*
      * Force the clipboard to be rendered if we are the clipboard owner.
      */
-    
+
     if (dispPtr->clipWindow) {
 	hwnd = Tk_GetHWND(Tk_WindowId(dispPtr->clipWindow));
 	if (GetClipboardOwner() == hwnd) {
@@ -459,6 +475,31 @@ XBell(display, percent)
     MessageBeep(MB_OK);
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * XLowerWindow --
+ *
+ *	Change the stacking order of a window.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Changes the stacking order of the specified window.
+ *
+ *----------------------------------------------------------------------
+ */
+
+#if 0 /* Better one in tkWinWindow.c */
+void
+XLowerWindow(display, w)
+    Display* display;
+    Window w;
+{
+}
+#endif
+
 /*
  *----------------------------------------------------------------------
  *
@@ -616,6 +657,7 @@ Tk_TranslateWinEvent(hwnd, message, wParam, lParam, resultPtr)
 	case WM_SYSKEYUP:
 	case WM_KEYDOWN:
 	case WM_KEYUP:
+	case WM_MOUSEWHEEL:
  	    GenerateXEvent(hwnd, message, wParam, lParam);
 	    return 1;
 	case WM_MENUCHAR:
@@ -694,7 +736,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	case WM_SETFOCUS:
 	case WM_KILLFOCUS: {
 	    TkWindow *otherWinPtr = (TkWindow *)Tk_HWNDToWindow((HWND) wParam);
-	    
+	
 	    /*
 	     * Compare toplevel windows to avoid reporting focus
 	     * changes within the same toplevel.
@@ -726,7 +768,14 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 		Tk_InternAtom((Tk_Window)winPtr, "CLIPBOARD");
 	    event.xselectionclear.time = TkpGetMS();
 	    break;
-	    
+	
+	case WM_MOUSEWHEEL:
+	    /*
+	     * The mouse wheel event is closer to a key event than a
+	     * mouse event in that the message is sent to the window
+	     * that has focus.
+	     */
+	
 	case WM_CHAR:
 	case WM_SYSKEYDOWN:
 	case WM_SYSKEYUP:
@@ -741,7 +790,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	    /*
 	     * Compute the screen and window coordinates of the event.
 	     */
-	    
+	
 	    msgPos = GetMessagePos();
 	    rootPoint = MAKEPOINTS(msgPos);
 	    clientPoint.x = rootPoint.x;
@@ -768,6 +817,18 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	     */
 
 	    switch (message) {
+		case WM_MOUSEWHEEL:
+		    /*
+		     * We have invented a new X event type to handle
+		     * this event.  It still uses the KeyPress struct.
+		     * However, the keycode field has been overloaded
+		     * to hold the zDelta of the wheel.
+		     */
+		
+		    event.type = MouseWheelEvent;
+		    event.xany.send_event = -1;
+		    event.xkey.keycode = (short) HIWORD(wParam);
+		    break;
 		case WM_SYSKEYDOWN:
 		case WM_KEYDOWN:
 		    /*
@@ -837,7 +898,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
  *
  * GetState --
  *
- *	This function constructs a state mask for the mouse buttons 
+ *	This function constructs a state mask for the mouse buttons
  *	and modifier keys as they were before the event occured.
  *
  * Results:
@@ -931,7 +992,7 @@ GetTranslatedKey(xkey)
     XKeyEvent *xkey;
 {
     MSG msg;
-    
+
     xkey->nchars = 0;
 
     while (xkey->nchars < XMaxTransChars
